﻿using System.Net;
using DTO;
using Entity;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Shared;

namespace WfApi.Controllers
{
    [ApiController]
    [Route("api/v1/character")] //Version API
    public class CharacterController : ControllerBase
    {
        private readonly ICharacterService<CharacterDTO> _character;

        private readonly ILogger<CharacterController> _logger;

        public CharacterController(ICharacterService<CharacterDTO> characterService, ILogger<CharacterController> logger)
        {
            _character = characterService;
            _logger = logger;

        }

        [HttpGet("{id}")] // Indiquer que l'id est dans l'URL
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status204NoContent)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        public async Task<IActionResult> GetCharacter(int id)
        {
            try
            {
                try
                {
                    var character = await _character.GetCharById(id);
                    return Ok(character);
                }
                catch(KeyNotFoundException e)
                {
                    return NotFound();
                }
            }
            catch (Exception e)
            {
                return StatusCode((int)HttpStatusCode.InternalServerError, new { message = "Internal Server Error (" + e + ")" });
            }
        }

        [HttpGet("all")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status204NoContent)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public async Task<IActionResult> GetAllSource(int index = 0, int count = 10)
        {
            try
            {
                var result = await _character.GetSomeChar(index, count);

                if (result != null)
                {
                    return await Task.FromResult<IActionResult>(Ok(result));
                }
                else
                {
                    return NoContent();
                }
            }
            catch (Exception e)
            {
                return StatusCode((int)HttpStatusCode.InternalServerError, new { message = "Internal Server Error (" + e + ")" });
            }
        }

        [HttpPost]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status204NoContent)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        public async Task<IActionResult> CreateCharacter([FromBody] CharacterDTO newCharacter)
        {
            try
            {
                if (newCharacter == null)
                {
                    return BadRequest(new { message = "Source data is required." });
                }
                try
                {
                    var existingSource = await _character.GetCharById(newCharacter.Id);
                    return Conflict(new { message = "A source with this ID already exists." });
                }
                catch (KeyNotFoundException e)
                {
                    await _character.AddCharacter(newCharacter);
                    return CreatedAtAction(nameof(GetAllSource), new { id = newCharacter.Id }, newCharacter);
                }
            }
            catch (Exception e)
            {
                return StatusCode((int)HttpStatusCode.InternalServerError, new { message = "Internal Server Error (" + e + ")" });
            }
        }

        [HttpPut()]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status204NoContent)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public async Task<IActionResult> UpdateCharacter([FromQuery] int id, [FromBody] CharacterDTO updatedCharacter)
        {
            try
            {
                if (updatedCharacter == null)
                {
                    return BadRequest(new { message = "new source data is required." });
                }

                var result = _character.UpdateCharacter(id, updatedCharacter);

                return Ok(result);
            }
            catch (Exception e)
            {
                return StatusCode((int)HttpStatusCode.InternalServerError, new { message = "Internal Server Error (" + e + ")" });
            }
        }
    }
}
